(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory() :
  typeof define === 'function' && define.amd ? define(factory) :
  (global.TagInputControl = factory());
}(this, (function () { 'use strict';

  function _typeof(obj) {
    if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") {
      _typeof = function (obj) {
        return typeof obj;
      };
    } else {
      _typeof = function (obj) {
        return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj;
      };
    }

    return _typeof(obj);
  }

  function _classCallCheck(instance, Constructor) {
    if (!(instance instanceof Constructor)) {
      throw new TypeError("Cannot call a class as a function");
    }
  }

  function _defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
      var descriptor = props[i];
      descriptor.enumerable = descriptor.enumerable || false;
      descriptor.configurable = true;
      if ("value" in descriptor) descriptor.writable = true;
      Object.defineProperty(target, descriptor.key, descriptor);
    }
  }

  function _createClass(Constructor, protoProps, staticProps) {
    if (protoProps) _defineProperties(Constructor.prototype, protoProps);
    if (staticProps) _defineProperties(Constructor, staticProps);
    return Constructor;
  }

  function _readOnlyError(name) {
    throw new Error("\"" + name + "\" is read-only");
  }

  function _slicedToArray(arr, i) {
    return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest();
  }

  function _arrayWithHoles(arr) {
    if (Array.isArray(arr)) return arr;
  }

  function _iterableToArrayLimit(arr, i) {
    var _arr = [];
    var _n = true;
    var _d = false;
    var _e = undefined;

    try {
      for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) {
        _arr.push(_s.value);

        if (i && _arr.length === i) break;
      }
    } catch (err) {
      _d = true;
      _e = err;
    } finally {
      try {
        if (!_n && _i["return"] != null) _i["return"]();
      } finally {
        if (_d) throw _e;
      }
    }

    return _arr;
  }

  function _nonIterableRest() {
    throw new TypeError("Invalid attempt to destructure non-iterable instance");
  }

  function addEvent(eventType, element, handler) {
    element.addEventListener(eventType, handler, false);
  }

  function removeEvent(eventType, element, handler) {
    element.removeEventListener(eventType, handler, false);
  }

  function parseDOMString(str) {
    var type = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'text/html';
    var parser = new DOMParser();
    var elDoc = parser.parseFromString(str, type);
    return elDoc.firstChild;
  }

  function createElement(elementName) {
    var props = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    var element = document.createElement(elementName);
    Object.entries(props).forEach(function (_ref) {
      var _ref2 = _slicedToArray(_ref, 2),
          key = _ref2[0],
          val = _ref2[1];

      element[key] = val;
    });
    return element;
  }

  function removeElement(element) {
    var events = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    Object.entries(events).forEach(function (_ref3) {
      var _ref4 = _slicedToArray(_ref3, 2),
          key = _ref4[0],
          val = _ref4[1];

      removeEvent(key, element, val);
    });
    element.parentNode.removeChild(element);
    element = null;
  }

  function hasText(element, tag) {
    return element.textContent === tag;
  }

  function queryElement(element) {
    var queryResult = undefined;

    if (element !== null && _typeof(element) === 'object' && element.nodeType === 1 && typeof element.nodeName === 'string') {
      queryResult = element;
    } else if (typeof element === 'string') {
      queryResult = document.getElementById(element);
    }

    if (!queryResult) {
      throw new Error('Invalid element');
    }

    return queryResult;
  }

  var defaultOptions = {
    cssClass: {
      tagPanel: 'tagi__panel',
      tagElement: 'tagi__element',
      inputElement: 'tagi__input'
    },
    renderRemoveButton: function renderRemoveButton() {
      return parseDOMString("<svg width=\"16\" height=\"16\" viewBox=\"0 0 16 16\" fill=\"none\" xmlns=\"http://www.w3.org/2000/svg\">\n      <path d=\"M13.5 2.5L2.5 13.5\" stroke=\"#454647\" stroke-miterlimit=\"10\" stroke-linecap=\"round\" stroke-linejoin=\"round\"/>\n      <path d=\"M2.5 2.5L13.5 13.5\" stroke=\"#454647\" stroke-miterlimit=\"10\" stroke-linecap=\"round\" stroke-linejoin=\"round\"/>\n      </svg>\n      ", 'image/svg+xml');
    },
    setFocus: false,
    initialTags: [],
    onTagChange: function onTagChange() {}
  };

  var TagPanel =
  /*#__PURE__*/
  function () {
    function TagPanel(element) {
      var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

      _classCallCheck(this, TagPanel);

      this.options = Object.assign({}, defaultOptions, options);
      this.options.cssClass = Object.assign({}, defaultOptions.cssClass, options.cssClass);
      this.panel = this._createPanel(element, this.options.cssClass.tagPanel);
      this.input = this._createInput(this.panel, {
        className: this.options.cssClass.inputElement,
        tabIndex: 0
      });
      this._subscribeEvents = this._subscribeEvents.bind(this);
      this._unsubscribeEvents = this._unsubscribeEvents.bind(this);
      this._handleTagInput = this._handleTagInput.bind(this);
      this._handleTagBlur = this._handleTagBlur.bind(this);
      this._handleTagClick = this._handleTagClick.bind(this);
      this._handleTagDelete = this._handleTagDelete.bind(this);
      this._handleGlobalKeyup = this._handleGlobalKeyup.bind(this);
      this._notifyCallbackFn = this._notifyCallbackFn.bind(this);
      this._initTags = this._initTags.bind(this);
      this.addTag = this.addTag.bind(this);
      this.destroy = this.destroy.bind(this);
      this.getTagList = this.getTagList.bind(this);
      this.removeTag = this.removeTag.bind(this);

      this._subscribeEvents(this);

      this._initTags();

      if (this.options.setFocus) {
        this.input.focus();
      }
    }

    _createClass(TagPanel, [{
      key: "destroy",
      value: function destroy() {
        this._unsubscribeEvents();

        removeElement(this.panel);
      }
    }, {
      key: "getTagList",
      value: function getTagList() {
        var result = [];

        if (this.panel.hasChildNodes()) {
          var nodes = this.panel.childNodes;

          for (var i = 0; i < nodes.length; i++) {
            var tag = this._getTag(nodes[i], this.options.cssClass.tagElement);

            if (tag) {
              result.push(tag);
            }
          }
        }

        return result;
      }
    }, {
      key: "addTag",
      value: function addTag(text) {
        var _this = this;

        this.panel.insertBefore(this._createTagElement(text), this.input);
        setTimeout(function () {
          return _this._notifyCallbackFn();
        });
      }
    }, {
      key: "removeTag",
      value: function removeTag(tagElement) {
        var _this2 = this;

        if (tagElement instanceof HTMLElement) {
          removeElement(tagElement);
          setTimeout(function () {
            return _this2._notifyCallbackFn();
          });
        } else if (typeof tagElement === 'string') {
          var result = [];
          this.panel.childNodes.forEach(function (el) {
            if (hasText(el, tagElement)) {
              result.push(el);
            }
          });
          result.forEach(function (el) {
            return removeElement(el);
          });
          result = (_readOnlyError("result"), null);
          setTimeout(function () {
            return _this2._notifyCallbackFn();
          });
        }
      }
    }, {
      key: "_initTags",
      value: function _initTags() {
        var _this3 = this;

        this.options.initialTags.forEach(function (tag) {
          return _this3.addTag(tag);
        });
      }
    }, {
      key: "_notifyCallbackFn",
      value: function _notifyCallbackFn() {
        this.options.onTagChange(this.getTagList());
      }
    }, {
      key: "_createTagElement",
      value: function _createTagElement(tag) {
        var tagEl = createElement('DIV', {
          className: this.options.cssClass.tagElement,
          tabIndex: this.panel.childNodes.length
        });
        var tagTextEl = createElement('SPAN', {
          innerHTML: tag
        });
        var tagIconEl = this.options.renderRemoveButton();
        addEvent('click', tagEl, this._handleTagClick);
        addEvent('click', tagIconEl, this._handleTagDelete);
        tagEl.appendChild(tagTextEl);
        tagEl.appendChild(tagIconEl);
        return tagEl;
      }
    }, {
      key: "_getTag",
      value: function _getTag(node, cssClass) {
        var text = '';
        var i = 0;

        if (node.nodeType === 1 && node.nodeName === 'DIV' && node.className === cssClass && node.hasChildNodes()) {
          while (node.childNodes[i]) {
            if (node.childNodes[i].nodeType === 1 && node.childNodes[i].nodeName === 'SPAN') {
              text = (node.childNodes[i].innerText || '').trim();
              i = -1;
            } else {
              i++;
            }
          }
        }

        return text;
      }
    }, {
      key: "_createPanel",
      value: function _createPanel(element, className) {
        var panel = createElement('DIV', {
          className: className
        });
        queryElement(element).appendChild(panel);
        return panel;
      }
    }, {
      key: "_createInput",
      value: function _createInput(panel, props) {
        var input = createElement('INPUT', props);
        input.setAttribute('type', 'text');
        panel.appendChild(input);
        return input;
      }
    }, {
      key: "_subscribeEvents",
      value: function _subscribeEvents() {
        addEvent('keyup', this.input, this._handleTagInput);
        addEvent('blur', this.input, this._handleTagBlur);
        addEvent('keyup', document, this._handleGlobalKeyup);
      }
    }, {
      key: "_unsubscribeEvents",
      value: function _unsubscribeEvents() {
        removeEvent('keyup', this.input, this._handleTagInput);
        removeEvent('blur', this.input, this._handleTagBlur);
        removeEvent('keyup', document, this._handleGlobalKeyup);
      }
    }, {
      key: "_handleTagBlur",
      value: function _handleTagBlur() {
        this.beingUpdate = null;
      }
    }, {
      key: "_handleTagClick",
      value: function _handleTagClick(e) {
        e.stopPropagation();
        this.beingUpdate = e.target;
        this.input.value = this.beingUpdate.innerText;
        this.input.focus();
      }
    }, {
      key: "_handleTagDelete",
      value: function _handleTagDelete(e) {
        e.stopPropagation();
        this.beingUpdate = null;
        this.removeTag(e.target.parentNode);
      }
    }, {
      key: "_handleTagInput",
      value: function _handleTagInput(e) {
        e.stopPropagation();
        var code = e.keyCode ? e.keyCode : e.which;

        if (code === 13 && e.target.value.length) {
          if (this.beingUpdate) {
            this.beingUpdate.innerText = e.target.value;
            this.beingUpdate = null;
          } else {
            this.addTag(e.target.value.trim());
          }

          window.setTimeout(function () {
            this.target.value = '';
          }.bind(e));
          return;
        } else if (code === 8 && !e.target.value.length) {
          var tagEl = this._getTheLastTag(this.panel, this.options.cssClass.tagElement);

          if (tagEl) {
            if (this.beingUpdate) {
              removeElement(this.beingUpdate.parentNode);
              return;
            }

            tagEl.focus();
            return;
          }
        } else if (code === 27) {
          this.beingUpdate = null;
          e.target.value = '';
        }

        if (this.beingUpdate) {
          this.beingUpdate.innerText = e.target.value;
        }
      }
    }, {
      key: "_handleGlobalKeyup",
      value: function _handleGlobalKeyup(e) {
        var code = e.keyCode ? e.keyCode : e.which;

        if (code === 8 && document.activeElement.parentNode === this.panel && document.activeElement.nodeType === 1 && document.activeElement.nodeName === 'DIV' && document.activeElement.className === this.options.cssClass.tagElement) {
          var focusedEl = document.activeElement;

          if (document.activeElement.previousSibling && document.activeElement.previousSibling.parentNode === this.panel) {
            document.activeElement.previousSibling.focus();
          }

          removeElement(focusedEl);
        } else if (code === 27) {
          this.beingUpdate = null;
          this.input.value = '';
        }
      }
    }, {
      key: "_getTheLastTag",
      value: function _getTheLastTag(panel, cssClass) {
        var i = panel.childNodes.length - 1 || -1;
        var node = null;

        while (node = panel.childNodes[i], node) {
          if (node.nodeType === 1 && node.nodeName === 'DIV' && node.className === cssClass) {
            return panel.childNodes[i];
          } else {
            i--;
          }
        }

        return null;
      }
    }]);

    return TagPanel;
  }();

  return TagPanel;

})));
